package com.ptashek.widgets.datetimepicker;

import java.util.Calendar;

import org.openmuc.android.R;

import android.app.Activity;
import android.app.Dialog;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.Button;
import android.widget.RelativeLayout;

public final class DateTimePickerDialog {

	public interface OnSubmitListener {
		void submit(Calendar result);
	}

	private final DateTimePicker mDateTimePicker;
	private final RelativeLayout mDateTimeDialogView;
	private final Dialog mDateTimeDialog;

	private final OnSubmitClickListener submitClickListener = new OnSubmitClickListener();
	private final OnResetClickListener resetClickListener = new OnResetClickListener();
	private final OnCancelClickListener cancelClickListener = new OnCancelClickListener();

	public DateTimePickerDialog(Activity ctx) {
		// Create the dialog
		mDateTimeDialog = new Dialog(ctx);
		// Inflate the root layout
		mDateTimeDialogView = (RelativeLayout) ctx.getLayoutInflater().inflate(R.layout.date_time_dialog, null);
		// Grab widget instance
		mDateTimePicker = (DateTimePicker) mDateTimeDialogView.findViewById(R.id.DateTimePicker);

		// Set the submit button to the desired behavior
		((Button) mDateTimeDialogView.findViewById(R.id.SetDateTime)).setOnClickListener(submitClickListener);
		// Cancel the dialog when the "Cancel" button is clicked
		((Button) mDateTimeDialogView.findViewById(R.id.CancelDialog)).setOnClickListener(cancelClickListener);
		// Reset Date and Time pickers when the "Reset" button is clicked
		((Button) mDateTimeDialogView.findViewById(R.id.ResetDateTime)).setOnClickListener(resetClickListener);

		// Check is system is set to use 24h time
		final String timeS = android.provider.Settings.System.getString(ctx.getContentResolver(),
				android.provider.Settings.System.TIME_12_24);
		final boolean is24h = !(timeS == null || timeS.equals("12"));
		mDateTimePicker.setIs24HourView(is24h);
		// No title on the dialog window
		mDateTimeDialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
		// Set the dialog content view
		mDateTimeDialog.setContentView(mDateTimeDialogView);
	}

	public void showDateTimeDialog(OnSubmitListener submitListener) {
		submitClickListener.listener = submitListener;
		mDateTimeDialog.show();
	}

	private class OnSubmitClickListener implements OnClickListener {
		private OnSubmitListener listener;

		@Override
		public void onClick(View v) {
			mDateTimePicker.clearFocus();
			Calendar result = Calendar.getInstance();
			result.setTimeInMillis(mDateTimePicker.getDateTimeMillis());
			listener.submit(result);
			mDateTimeDialog.dismiss();
		}
	}

	private class OnResetClickListener implements OnClickListener {
		@Override
		public void onClick(View v) {
			mDateTimePicker.reset();
		}
	}

	private class OnCancelClickListener implements OnClickListener {
		@Override
		public void onClick(View v) {
			mDateTimeDialog.cancel();
		}
	}
}
