package org.openmuc.android.activities;

import java.io.IOException;
import java.util.List;

import org.openmuc.android.R;
import org.openmuc.android.rest.RestClient;
import org.openmuc.android.rest.data.Channel;

import android.annotation.TargetApi;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceCategory;
import android.preference.PreferenceFragment;
import android.preference.PreferenceManager;
import android.support.v4.app.NavUtils;
import android.view.MenuItem;

/**
 * A {@link PreferenceActivity} that presents a set of application settings. On handset devices, settings are presented
 * as a single list. On tablets, settings are split by category, with category headers shown to the left of the list of
 * settings.
 * <p>
 * See <a href="http://developer.android.com/design/patterns/settings.html"> Android Design: Settings</a> for design
 * guidelines and the <a href="http://developer.android.com/guide/topics/ui/settings.html">Settings API Guide</a> for
 * more information on developing a Settings UI.
 */
public class SettingsActivity extends PreferenceActivity {
	/**
	 * Determines whether to always show the simplified settings UI, where settings are presented in a single list. When
	 * false, settings are shown as a master/detail two-pane view on tablets. When true, a single pane is shown on
	 * tablets.
	 */
	private static final boolean ALWAYS_SIMPLE_PREFS = false;

	// Static so sBindPreferenceSummaryToValueListener can access the value
	private static boolean serverConfigChanged = false;

	private ServerPingTask runningTask;
	private Dialog dialog;
	private boolean dialogCreated = false;

	@SuppressWarnings("deprecation")
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setupActionBar();

		Object retained = getLastNonConfigurationInstance();
		if (retained != null) {
			runningTask = (ServerPingTask) retained;
			runningTask.setActivity(this);
			if (!runningTask.completed) {
				runningTask.onPreExecute();
			}
		}

		if (savedInstanceState != null && savedInstanceState.containsKey("serverConfigChanged")) {
			serverConfigChanged = savedInstanceState.getBoolean("serverConfigChanged");
		}
		else {
			serverConfigChanged = false;
		}
	}

	@Override
	protected void onStart() {
		super.onStart();
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putBoolean("serverConfigChanged", serverConfigChanged);
	}

	/**
	 * Set up the {@link android.app.ActionBar}, if the API is available.
	 */
	@TargetApi(Build.VERSION_CODES.HONEYCOMB)
	private void setupActionBar() {
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
			// Show the Up button in the action bar.
			getActionBar().setDisplayHomeAsUpEnabled(true);
		}
	}

	/**
	 * Starts a AsyncTask to connect with the server and check for available channels, if the server address or port
	 * have been changed.
	 * 
	 * @return true if server address or port have been changed and a connect check is performed.
	 */
	private boolean processServerConfigChanged() {
		if (serverConfigChanged) {
			SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(SettingsActivity.this);
			final String serverAddress = prefs.getString(getString(R.string.preference_server_address), "");
			final String serverPort = prefs.getString(getString(R.string.preference_server_port), "");
			runningTask = new ServerPingTask(this);
			runningTask.execute(serverAddress + ":" + serverPort);

			return true;
		}
		return false;
	}

	@Override
	public void onBackPressed() {
		if (!processServerConfigChanged()) {
			super.onBackPressed();
		}
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// This ID represents the Home or Up button. In the case of this
			// activity, the Up button is shown. Use NavUtils to allow users
			// to navigate up one level in the application structure. For
			// more details, see the Navigation pattern on Android Design:
			//
			// http://developer.android.com/design/patterns/navigation.html#up-vs-back
			//
			// TODO: If Settings has multiple levels, Up should navigate up
			// that hierarchy.
			if (!processServerConfigChanged()) {
				NavUtils.navigateUpFromSameTask(this);
			}
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	protected void onPostCreate(Bundle savedInstanceState) {
		super.onPostCreate(savedInstanceState);

		setupSimplePreferencesScreen();
	}

	/**
	 * Shows the simplified settings UI if the device configuration if the device configuration dictates that a
	 * simplified, single-pane UI should be shown.
	 */
	@SuppressWarnings("deprecation")
	private void setupSimplePreferencesScreen() {
		if (!isSimplePreferences(this)) {
			return;
		}

		// In the simplified UI, fragments are not used at all and we instead
		// use the older PreferenceActivity APIs.

		// Add 'general' preferences.
		setPreferenceScreen(getPreferenceManager().createPreferenceScreen(this));

		PreferenceCategory fakeHeader = new PreferenceCategory(this);
		fakeHeader.setTitle(R.string.pref_header_server);
		getPreferenceScreen().addPreference(fakeHeader);
		addPreferencesFromResource(R.xml.pref_server);

		// Add 'notifications' preferences, and a corresponding header.
		fakeHeader = new PreferenceCategory(this);
		fakeHeader.setTitle(R.string.pref_header_comm);
		getPreferenceScreen().addPreference(fakeHeader);
		addPreferencesFromResource(R.xml.pref_comm);

		fakeHeader = new PreferenceCategory(this);
		fakeHeader.setTitle(R.string.pref_header_layout);
		getPreferenceScreen().addPreference(fakeHeader);
		EditTextPreference item = new EditTextPreference(this);
		item.setTitle(R.string.pref_title_y_axis_min);
		item.setKey(getString(R.string.pref_key_y_axis_min).toString());
		fakeHeader.addPreference(item);
		item = new EditTextPreference(this);
		item.setTitle(R.string.pref_title_y_axis_max);
		item.setKey(getString(R.string.pref_key_y_axis_max).toString());
		fakeHeader.addPreference(item);

		// Bind the summaries of EditText/List/Dialog/Ringtone preferences to
		// their values. When their values change, their summaries are updated
		// to reflect the new value, per the Android Design guidelines.
		bindPreferenceSummaryToValue(findPreference(getText(R.string.preference_server_address)));
		bindPreferenceSummaryToValue(findPreference(getText(R.string.preference_server_port)));
		bindPreferenceSummaryToValue(findPreference(getText(R.string.pref_key_update_interval)));
		bindPreferenceSummaryToValue(findPreference(getText(R.string.pref_key_y_axis_min)));
		bindPreferenceSummaryToValue(findPreference(getText(R.string.pref_key_y_axis_max)));
	}

	/** {@inheritDoc} */
	@Override
	public boolean onIsMultiPane() {
		return isLargeTablet(this) && !isSimplePreferences(this);
	}

	/**
	 * Helper method to determine if the device has an extra-large screen. For example, 10" tablets are extra-large.
	 */
	private static boolean isLargeTablet(Context context) {
		int width = context.getResources().getDisplayMetrics().widthPixels;
		int height = context.getResources().getDisplayMetrics().heightPixels;
		int min = Math.min(width, height);

		return min >= 600;
	}

	/**
	 * Determines whether the simplified settings UI should be shown. This is true if this is forced via
	 * {@link #ALWAYS_SIMPLE_PREFS}, or the device doesn't have newer APIs like {@link PreferenceFragment}, or the
	 * device doesn't have an extra-large screen. In these cases, a single-pane "simplified" settings UI should be
	 * shown.
	 */
	private static boolean isSimplePreferences(Context context) {
		return ALWAYS_SIMPLE_PREFS || Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB || !isLargeTablet(context);
	}

	/** {@inheritDoc} */
	@Override
	@TargetApi(Build.VERSION_CODES.HONEYCOMB)
	public void onBuildHeaders(List<Header> target) {
		if (!isSimplePreferences(this)) {
			loadHeadersFromResource(R.xml.pref_headers, target);
		}
	}

	/**
	 * Callback used by ServerPingTask to notify the activity of a completed server request. Depending on the result of
	 * the task, this activity is either closed or a confirmation dialog opens to ensure that the user wants to use a
	 * server address that has no channels
	 * 
	 * @param taskId
	 *            Id of the finished task
	 */
	public void onTaskCompleted(int taskId) {
		if (taskId == ServerPingTask.TASK_ID) {
			if (runningTask.wasSuccess()) {
				NavUtils.navigateUpFromSameTask(SettingsActivity.this);
			}
			else {
				AlertDialog.Builder builder = new AlertDialog.Builder(SettingsActivity.this);
				builder.setTitle("Warning").setMessage(
						"No channels found on server " + runningTask.getServerAddress()
								+ "\nDo you want to continue anyway?");
				builder.setPositiveButton("Yes", new OnClickListener() {
					@Override
					public void onClick(DialogInterface paramDialogInterface, int paramInt) {
						paramDialogInterface.dismiss();
						NavUtils.navigateUpFromSameTask(SettingsActivity.this);
					}
				});
				builder.setNegativeButton("No", new OnClickListener() {
					@Override
					public void onClick(DialogInterface paramDialogInterface, int paramInt) {
						paramDialogInterface.dismiss();
					}
				});
				builder.create().show();
			}
		}

		if (dialogCreated) {
			dialog.dismiss();
		}
		dialog = null;
		runningTask = null;
	}

	@Override
	public Object onRetainNonConfigurationInstance() {
		if (runningTask != null) {
			runningTask.setActivity(null);
		}
		return runningTask;
	}

	/**
	 * A preference value change listener that updates the preference's summary to reflect its new value.
	 */
	private static Preference.OnPreferenceChangeListener sBindPreferenceSummaryToValueListener = new Preference.OnPreferenceChangeListener() {
		@Override
		public boolean onPreferenceChange(Preference preference, Object value) {
			String stringValue = value.toString();

			Context ctx = preference.getContext();
			if ((preference.getTitle().equals(ctx.getString(R.string.preference_server_address_label)) || preference
					.getTitle().equals(ctx.getString(R.string.preference_server_port_label)))
					&& preference.getSummary() != null && preference.getSummary().equals(stringValue) == false) {
				serverConfigChanged = true;
			}
			// For all other preferences, set the summary to the value's
			// simple string representation.
			preference.setSummary(stringValue);

			return true;
		}
	};

	/**
	 * Binds a preference's summary to its value. More specifically, when the preference's value is changed, its summary
	 * (line of text below the preference title) is updated to reflect the value. The summary is also immediately
	 * updated upon calling this method. The exact display format is dependent on the type of preference.
	 * 
	 * @see #sBindPreferenceSummaryToValueListener
	 */
	private static void bindPreferenceSummaryToValue(Preference preference) {
		if (preference != null) {
			// Set the listener to watch for value changes.
			preference.setOnPreferenceChangeListener(sBindPreferenceSummaryToValueListener);

			// Trigger the listener immediately with the preference's
			// current value.
			if (preference instanceof CheckBoxPreference) {
				sBindPreferenceSummaryToValueListener.onPreferenceChange(preference, preference.getSharedPreferences()
						.getBoolean(preference.getKey(), false));
			}
			else {
				sBindPreferenceSummaryToValueListener.onPreferenceChange(preference, PreferenceManager
						.getDefaultSharedPreferences(preference.getContext()).getString(preference.getKey(), ""));
			}
		}
	}

	/**
	 * This fragment shows server preferences only. It is used when the activity is showing a two-pane settings UI.
	 */
	@TargetApi(Build.VERSION_CODES.HONEYCOMB)
	public static class ServerPreferenceFragment extends PreferenceFragment {
		@Override
		public void onCreate(Bundle savedInstanceState) {
			super.onCreate(savedInstanceState);
			addPreferencesFromResource(R.xml.pref_server);

			// Bind the summaries of EditText/List/Dialog/Ringtone preferences
			// to their values. When their values change, their summaries are
			// updated to reflect the new value, per the Android Design
			// guidelines.
			bindPreferenceSummaryToValue(findPreference(getString(R.string.preference_server_address)));
			bindPreferenceSummaryToValue(findPreference(getString(R.string.preference_server_port)));
		}
	}

	/**
	 * This fragment shows connection preferences only. It is used when the activity is showing a two-pane settings UI.
	 */
	@TargetApi(Build.VERSION_CODES.HONEYCOMB)
	public static class NotificationPreferenceFragment extends PreferenceFragment {
		@Override
		public void onCreate(Bundle savedInstanceState) {
			super.onCreate(savedInstanceState);
			addPreferencesFromResource(R.xml.pref_comm);

			// Bind the summaries of EditText/List/Dialog/Ringtone preferences
			// to their values. When their values change, their summaries are
			// updated to reflect the new value, per the Android Design
			// guidelines.
			bindPreferenceSummaryToValue(findPreference(getString(R.string.pref_key_update_interval)));
		}
	}

	/**
	 * This fragment shows layout preferences only. It is used when the activity is showing a two-pane settings UI.
	 */
	@TargetApi(Build.VERSION_CODES.HONEYCOMB)
	public static class DataSyncPreferenceFragment extends PreferenceFragment {
		@Override
		public void onCreate(Bundle savedInstanceState) {
			super.onCreate(savedInstanceState);
			addPreferencesFromResource(R.xml.pref_layout);

			// Bind the summaries of EditText/List/Dialog/Ringtone preferences
			// to their values. When their values change, their summaries are
			// updated to reflect the new value, per the Android Design
			// guidelines.
			bindPreferenceSummaryToValue(findPreference(getString(R.string.pref_key_showChannels)));
			bindPreferenceSummaryToValue(findPreference(getString(R.string.pref_key_y_axis_min)));
			bindPreferenceSummaryToValue(findPreference(getString(R.string.pref_key_y_axis_max)));
		}
	}

	/**
	 * AsyncTask that connects to the server with the provided address and checks for available channels
	 * 
	 * @author Karsten Müller-Bier
	 */
	private static class ServerPingTask extends AsyncTask<String, Void, Boolean> {
		private static final int TASK_ID = 1;

		private SettingsActivity activity;
		private boolean completed;
		private String serverAddress;

		private boolean success;

		private ServerPingTask(SettingsActivity activity) {
			this.activity = activity;
		}

		/**
		 * Method to check if the completed task found channels on the server
		 * 
		 * @return true if the task was able to connect to the server and at least one channel is available
		 */
		public boolean wasSuccess() {
			return success;
		}

		public String getServerAddress() {
			return serverAddress;
		}

		/**
		 * Showing the dialog on the UI thread.
		 */
		@Override
		protected void onPreExecute() {
			activity.dialog = ProgressDialog.show(activity, "Please wait", "Trying connection settings", false);
			activity.dialogCreated = true;
		}

		/**
		 * Performing the heavy lifting in the background thread thread.
		 */
		@Override
		protected Boolean doInBackground(String... address) {
			serverAddress = address[0];
			RestClient client = new RestClient(address[0]);
			List<Channel> channels = null;
			try {
				channels = client.listChannels();
			} catch (IOException e) {
			}

			return !(channels == null || channels.isEmpty());
		}

		/**
		 * When the task is completed, notifiy the Activity.
		 */
		@Override
		protected void onPostExecute(Boolean result) {
			this.success = result;
			completed = true;
			notifyActivityTaskCompleted();
		}

		/**
		 * Notify the task that the activity has been destroyed and created anew, possibly due to rotation of the
		 * device. If the task has been completed while the activity has been recreated, the callback Method of the
		 * activity is called directly.
		 * 
		 * @param activity
		 *            New reference to the SettingsActivity instance
		 */
		private void setActivity(SettingsActivity activity) {
			this.activity = activity;
			if (completed) {
				notifyActivityTaskCompleted();
			}
		}

		/**
		 * Helper method to notify the activity that this task completed.
		 */
		private void notifyActivityTaskCompleted() {
			if (null != activity) {
				activity.onTaskCompleted(TASK_ID);
			}
		}

	}
}
