package org.openmuc.android.activities.handheld;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.openmuc.android.R;
import org.openmuc.android.rest.data.Channel;
import org.openmuc.android.util.ChannelAdapter;
import org.openmuc.android.util.SharedPreferenceUtil;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

/**
 * Support activity used on devices running on Android version 2.3.3 or lower, to give users the ability to select
 * channels that shall be removed from the observed channels.
 * 
 * On devices running on an Android version >= 3.0, this activity is not called. Instead, the choice mode of the
 * {@link ChannelList} activity is activated and used if a channel is pressed long enough.
 * 
 * @author Karsten Müller-Bier
 */
public class DeleteChannelActivity extends Activity {

	public static final int REQUEST_CODE = 2;

	List<Channel> channelList;
	ChannelAdapter adapter;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_add_channel);
	}

	@Override
	protected void onStart() {
		super.onStart();

		channelList = new ArrayList<Channel>();
		Set<String> labels = SharedPreferenceUtil.getChannelLabels(this);
		for (String label : labels) {
			Channel newChannel = new Channel();
			newChannel.setLabel(label);
			newChannel.getRecord().setFlag("");
			channelList.add(newChannel);
		}

		adapter = new ChannelAdapter(this, R.layout.selectable_row, channelList);
		ListView view = (ListView) findViewById(R.id.addChannelList);
		view.setAdapter(adapter);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.remove_channel, menu);
		return true;
	}

	@Override
	public void onBackPressed() {
		setResult(RESULT_CANCELED);
		finish();
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		Intent returnIntent = new Intent();
		switch (item.getItemId()) {
		case R.id.action_accept:
			Set<String> removedChannels = getCheckedChannels();
			returnIntent.putExtra("channelLabels", removedChannels.toArray(new String[0]));
			setResult(RESULT_OK, returnIntent);
			finish();
			return true;
		case R.id.action_abort:
			setResult(RESULT_CANCELED);
			finish();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	/**
	 * Traverses the channelList returns the label of all channels that were checked
	 * 
	 * @return Set of labels of all checked channels
	 */
	private Set<String> getCheckedChannels() {
		Set<String> result = new HashSet<String>();

		ListView view = (ListView) findViewById(R.id.addChannelList);
		int listSize = view.getChildCount();
		for (int i = 0; i < listSize; i++) {
			LinearLayout listItem = (LinearLayout) view.getChildAt(i);
			CheckBox check = (CheckBox) listItem.findViewById(R.id.channelselected);
			if (check.isChecked()) {
				TextView text = (TextView) listItem.findViewById(R.id.toptext);
				result.add(text.getText().toString());
			}
		}

		return result;
	}
}
