package org.openmuc.android.util;

import java.util.Collections;
import java.util.List;
import java.util.Locale;

import org.openmuc.android.R;
import org.openmuc.android.rest.data.Channel;

import android.content.Context;
import android.graphics.Typeface;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.TextView;

/**
 * ArrayAdapter that shows channels is ListItems with two text fields. The upper field consists of the channel label,
 * the lower field list the current known value
 * 
 * @author Karsten Müller-Bier
 */
public class ChannelAdapter extends ArrayAdapter<Channel> {

	protected List<Channel> channels;
	protected int layoutId;

	public ChannelAdapter(Context context, int layoutId, List<Channel> objects) {
		super(context, 0, objects);
		this.layoutId = layoutId;
		channels = objects;
	}

	public ChannelAdapter(Context context, List<Channel> objects) {
		this(context, R.layout.row, objects);
	}

	public List<Channel> getChannels() {
		return Collections.unmodifiableList(channels);
	}

	@Override
	public int getCount() {
		return channels.size();
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		View v = convertView;
		if (v == null) {
			LayoutInflater vi = (LayoutInflater) getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			v = vi.inflate(layoutId, null);
		}
		Channel channel = channels.get(position);
		if (channel != null) {
			TextView upper = (TextView) v.findViewById(R.id.toptext);
			TextView lower = (TextView) v.findViewById(R.id.bottomtext);
			if (upper != null) {
				upper.setTypeface(null, Typeface.BOLD);
				upper.setText(channel.getLabel());
			}
			if (lower != null) {
				if ("VALID".equals(channel.getRecord().getFlag().toUpperCase(Locale.US))) {
					lower.setText(channel.getRecord().getValue().toString());
				}
				else {
					lower.setText(channel.getRecord().getFlag());
				}
			}
		}

		return v;
	}

}
