package org.openmuc.android.util;

import java.util.HashSet;
import java.util.Set;

import org.openmuc.android.R;
import org.openmuc.android.activities.handheld.ChannelList;
import org.openmuc.android.activities.tablet.DashBoard;

import android.annotation.TargetApi;
import android.os.Build;
import android.view.ActionMode;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.AbsListView.MultiChoiceModeListener;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

/**
 * Overlay to choose channels that shall be removed from the list of observed channels <br>
 * <br>
 * This overlay is used by {@link DashBoard} and {@link ChannelList} activities on Android version 3.0 or higher
 * 
 * @author Karsten Müller-Bier
 */
@TargetApi(Build.VERSION_CODES.HONEYCOMB)
public class MainMultiChoiceModeListener implements MultiChoiceModeListener {

	private final ChannelAdapter origAdapter;
	private final ChannelAdapter checkedAdapter;
	private final ListView channelListView;
	private final ChannelDeleteChoiceCallback callback;

	public MainMultiChoiceModeListener(ChannelDeleteChoiceCallback callback, ListView channelList,
			ChannelAdapter adapter) {
		this.callback = callback;
		channelListView = channelList;
		origAdapter = adapter;
		checkedAdapter = new ChannelAdapter(adapter.getContext(), R.layout.selectable_row, adapter.getChannels());
	}

	@Override
	public void onItemCheckedStateChanged(ActionMode mode, int position, long id, boolean checked) {
		// Here you can do something when items are selected/de-selected,
		// such as update the title in the CAB
	}

	@Override
	public boolean onActionItemClicked(ActionMode mode, MenuItem item) {
		// Respond to clicks on the actions in the CAB
		switch (item.getItemId()) {
		case R.id.action_delete:
			Set<String> channelsToDelete = getCheckedChannels();
			callback.deleteChannels(channelsToDelete);
			mode.finish(); // Action picked, so close the CAB
			return true;
		default:
			return false;
		}
	}

	@Override
	public boolean onCreateActionMode(ActionMode mode, Menu menu) {
		// Inflate the menu for the CAB and change layout of the ListView to show a checkbox on each item
		MenuInflater inflater = mode.getMenuInflater();
		inflater.inflate(R.menu.main_action_bar, menu);
		channelListView.setAdapter(checkedAdapter);
		return true;
	}

	@Override
	public void onDestroyActionMode(ActionMode mode) {
		// return ListView to its original layout
		channelListView.setAdapter(origAdapter);
	}

	@Override
	public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
		// Here you can perform updates to the CAB due to
		// an invalidate() request
		return false;
	}

	private Set<String> getCheckedChannels() {
		Set<String> result = new HashSet<String>();

		int listSize = channelListView.getChildCount();
		for (int i = 0; i < listSize; i++) {
			LinearLayout listItem = (LinearLayout) channelListView.getChildAt(i);
			CheckBox check = (CheckBox) listItem.findViewById(R.id.channelselected);
			if (check.isChecked()) {
				TextView text = (TextView) listItem.findViewById(R.id.toptext);
				result.add(text.getText().toString());
			}
		}

		return result;
	}

	/**
	 * Callback interface to notify the creator object that channels have to be deleted
	 * 
	 * @author Karsten Müller-Bier
	 */
	public interface ChannelDeleteChoiceCallback {
		/**
		 * @param channelsToDelete
		 *            channel labels that shall be removed from the list of observed channels
		 */
		void deleteChannels(Set<String> channelsToDelete);
	}
}
