package org.openmuc.android.util;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.openmuc.android.R;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Build;
import android.preference.PreferenceManager;

@SuppressLint("NewApi")
/**
 * Helper class used to provide a way to load an array of stored values from the preference across different Android versions
 * 
 * @author Karsten Müller-Bier
 */
public class SharedPreferenceUtil {

	/**
	 * Loads all stored channels from the default preference
	 * 
	 * @param ctx
	 *            Context to load the channels
	 * @return set of all stored channel labels
	 */
	public static Set<String> getChannelLabels(Context ctx) {
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(ctx);

		Set<String> result = null;
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB) {
			result = new HashSet<String>();
			int size = prefs.getInt(ctx.getString(R.string.preference_registered_channels) + "_length", 0);
			for (int i = 0; i < size; i++) {
				String label = prefs.getString(ctx.getString(R.string.preference_registered_channels) + "_" + i, null);
				if (label != null) {
					result.add(label);
				}
			}
		}
		else {
			result = prefs.getStringSet(ctx.getString(R.string.preference_registered_channels), null);
			if (result == null) {
				result = new HashSet<String>();
			}
			else {
				result = new HashSet<String>(result);
			}
		}

		return result;
	}

	/**
	 * Writes the provided set of strings as channel labels into the default preferences, overwriting previously stored
	 * labels
	 * 
	 * @param ctx
	 *            Context to write labels
	 * @param labels
	 *            set of labels to write into preference
	 */
	public static void writeChannelLabels(Context ctx, Set<String> labels) {
		Editor editor = PreferenceManager.getDefaultSharedPreferences(ctx).edit();

		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB) {
			editor.putInt(ctx.getString(R.string.preference_registered_channels) + "_length", labels.size());
			Iterator<String> iter = labels.iterator();
			for (int i = 0; i < labels.size(); i++) {
				editor.putString(ctx.getString(R.string.preference_registered_channels) + "_" + i, iter.next());
			}
		}
		else {
			editor.putStringSet(ctx.getString(R.string.preference_registered_channels), labels);
		}

		editor.apply();
	}
}
