/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.openmuc.framework.config;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;

public interface ConfigService {

	public void lock();

	public boolean tryLock();

	public void unlock();

	public RootConfig getConfig();

	public RootConfig getConfig(ConfigChangeListener listener);

	public void stopListeningForConfigChange(ConfigChangeListener listener);

	public void setConfig(RootConfig config);

	public void writeConfigToFile() throws ConfigWriteException;

	public void reloadConfigFromFile() throws FileNotFoundException, ParseException;

	public RootConfig getEmptyConfig();

	public List<DeviceScanInformation> scanForDevices(String driverName, String interfaceAddress, String scanFilter,
			String settings, int timeout) throws DriverNotAvailableException, UnsupportedOperationException,
			ArgumentSyntaxException, IOException;

	public List<ChannelScanInformation> scanForChannels(String driverName, String interfaceAddress,
			String deviceAddress, String settings, int timeout) throws DriverNotAvailableException,
			UnsupportedOperationException, IOException;

	public List<String> getIdsOfRegisteredDrivers();

	public List<String> getIdsOfRegisteredDrivers(DriverChangeListener listener);
}
