/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.openmuc.framework.data;

/**
 * A Record may represent a reading or a database entry. Record is immutable. It contains a value, a timestamp, and a
 * flag. If a record represents a measurement reading then the timestamp is supplied by the communication driver that
 * retrieved the data. If the data record is a processed value (e.g. a mean value that was stored in the database) then
 * the timestamp is supplied by the OpenMUC core.
 */
public final class Record {

	private final Long timestamp;
	private final Flag flag;
	private final Value value;

	public Record(Value value, Long timestamp, Flag flag) {
		this.value = value;
		this.timestamp = timestamp;
		this.flag = flag;
	}

	public Record(Value value, Long timestamp) {
		this(value, timestamp, Flag.VALID);
	}

	public Value getValue() {
		return value;
	}

	public Long getTimestamp() {
		return timestamp;
	}

	public Flag getFlag() {
		return flag;
	}

	@Override
	public String toString() {
		return "value: " + value + "; timestamp: " + timestamp + "; flag: " + flag;
	}

}
