package org.openmuc.framework.core.datamanager;

import java.util.List;
import java.util.concurrent.CountDownLatch;

import org.openmuc.framework.data.Flag;
import org.openmuc.framework.data.Record;
import org.openmuc.framework.driver.spi.ChannelRecordContainer;
import org.openmuc.framework.driver.spi.ConnectionException;

public class ReadTask extends DeviceReadTask {

	private final CountDownLatch readTaskFinishedSignal;

	public ReadTask(DataManager dataManager, Device device, List<ChannelRecordContainerImpl> selectedChannels,
			int readTimeout, CountDownLatch readTaskFinishedSignal) {
		super(dataManager, device, selectedChannels, readTimeout);
		this.readTaskFinishedSignal = readTaskFinishedSignal;
	}

	@Override
	@SuppressWarnings("unchecked")
	protected void executeRead() throws UnsupportedOperationException, ConnectionException {
		device.deviceConfig.driverParent.activeDriver.read(device.connection,
				(List<ChannelRecordContainer>) ((List<?>) selectedChannels), true, null, readTimeout);
	}

	@Override
	protected void taskFinished() {
		disabled = true;
		long now = System.currentTimeMillis();
		if (methodNotExceptedExceptionThrown) {
			for (ChannelRecordContainerImpl driverChannel : selectedChannels) {
				driverChannel.setRecord(new Record(null, now, Flag.ACCESS_METHOD_NOT_SUPPORTED));
			}
		}
		else if (unknownDriverExceptionThrown) {
			for (ChannelRecordContainerImpl driverChannel : selectedChannels) {
				driverChannel.setRecord(new Record(null, now, Flag.DRIVER_THREW_UNKNOWN_EXCEPTION));
			}
		}

		readTaskFinishedSignal.countDown();

		synchronized (dataManager.tasksFinished) {
			dataManager.tasksFinished.add(this);
		}
		dataManager.interrupt();
	}

	@Override
	protected void taskAborted() {
		readTaskFinishedSignal.countDown();
	}
}
