/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.openmuc.framework.core.datamanager;

import java.util.List;

import org.openmuc.framework.data.Flag;
import org.openmuc.framework.driver.spi.ChannelRecordContainer;
import org.openmuc.framework.driver.spi.ConnectionException;

public final class SamplingTask extends DeviceReadTask {

	String samplingGroup;

	public SamplingTask(DataManager dataManager, Device device, List<ChannelRecordContainerImpl> selectedChannels,
			int readTimeout, String samplingGroup) {
		super(dataManager, device, selectedChannels, readTimeout);
		this.samplingGroup = samplingGroup;
	}

	public void storeValues() {
		if (disabled) {
			return;
		}
		if (methodNotExceptedExceptionThrown) {
			for (ChannelRecordContainerImpl driverChannel : selectedChannels) {
				driverChannel.channel.setFlag(Flag.ACCESS_METHOD_NOT_SUPPORTED);
			}
		}
		else if (unknownDriverExceptionThrown) {
			for (ChannelRecordContainerImpl driverChannel : selectedChannels) {
				driverChannel.channel.setFlag(Flag.DRIVER_THREW_UNKNOWN_EXCEPTION);
			}
		}
		else {
			for (ChannelRecordContainerImpl driverChannel : selectedChannels) {
				driverChannel.channel.setNewSample(driverChannel.record);
			}
		}
		disabled = true;

	}

	@SuppressWarnings("unchecked")
	@Override
	protected void executeRead() throws UnsupportedOperationException, ConnectionException {
		device.deviceConfig.driverParent.activeDriver.read(device.connection,
				(List<ChannelRecordContainer>) ((List<?>) selectedChannels), null, samplingGroup, readTimeout);
	}

	@Override
	protected void taskFinished() {
		for (ChannelRecordContainerImpl channel : selectedChannels) {
			channel.channel.handle = channel.getChannelHandle();
		}

		synchronized (dataManager.samplingTaskFinished) {
			dataManager.samplingTaskFinished.add(this);
		}
		dataManager.interrupt();
	}

	@Override
	protected void taskAborted() {
	}
}
