/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.openmuc.framework.datalogger.spi;

import java.io.IOException;
import java.util.List;

import org.openmuc.framework.data.Record;

public interface DataLoggerService {

	public String getId();

	public void setChannelsToLog(List<LogChannel> channels);

	public void log(List<LogRecordContainer> containers, long timestamp);

	/**
	 * Gets all values (time series) of a channel from startTime until now. It is the same as the call getValues(label,
	 * startTime, System.currentTimeMillies)
	 * 
	 * @param id
	 *            unique identifier of the item
	 * @param startTimestamp
	 *            Time of the first value in the time series in ms since epoche.
	 * @return A List of value objects or an empty list if now matching object have been found.
	 */
	public List<Record> getValues(String id, long startTimestamp) throws IOException;

	/**
	 * Gets all values (time series) of a channel from startTime until endTime.
	 * 
	 * @param id
	 *            unique identifier of the item
	 * @param startTimestamp
	 *            Time of the first value in the time series in ms since epoche.
	 * @param endTimestamp
	 *            Time of the last value in the time series in ms since epoche
	 * @return A List of value objects or an empty list if now matching object have been found.
	 */
	public List<Record> getValues(String id, long startTimestamp, long endTimestamp) throws IOException;

	/**
	 * Gets a single value with a specified timestamp from the database.
	 * 
	 * @param id
	 *            unique identifier of the item
	 * @param timestamp
	 *            timestamp of the value (ms since epoche)
	 * @return a Value object if to has been found or null
	 */
	public Record getValue(String id, long timestamp) throws IOException;

}
