/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openmuc.driver.dlms;

import org.openmuc.jdlms.client.api.GetParameter;

public class ChannelAddress {

	private final int classId;
	private final String instanceId;
	private final String printableInstanceId;
	private final int attributeId;

	public ChannelAddress(int classId, String instanceId, int attributeId) {
		this.classId = classId;
		printableInstanceId = instanceId;
		this.instanceId = instanceId;
		this.attributeId = attributeId;
	}

	public ChannelAddress(int classId, byte[] instanceId, int attributeId) {
		this.classId = classId;
		printableInstanceId = parseInstanceId(instanceId);
		this.instanceId = parseInstanceId(printableInstanceId);
		this.attributeId = attributeId;
	}

	public static ChannelAddress parse(String input) {
		String[] tokens = input.split("/");
		if (tokens.length != 3) {
			throw new IllegalArgumentException("Channel address must be of format 'classId/logicalName/attributeId'");
		}

		int classId = Integer.parseInt(tokens[0]);
		int attributeId = Integer.parseInt(tokens[2]);
		String instanceId = parseInstanceId(tokens[1]);

		return new ChannelAddress(classId, instanceId, attributeId);
	}

	private static String parseInstanceId(String idString) {
		String result = null;

		String tokens[] = idString.split("\\.");

		if (tokens.length == 1) {
			result = idString;
		}
		else if (tokens.length == 6) {

			StringBuilder sb = new StringBuilder(12);
			for (String byteString : tokens) {
				int val = Integer.parseInt(byteString);
				if (val < 16) {
					sb.append("0");
				}
				sb.append(Integer.toHexString(val));
			}
			result = sb.toString();
		}
		else {
			throw new IllegalArgumentException("Reduced ID codes are not supported");
		}

		return result;
	}

	private static String parseInstanceId(byte[] instanceId) {
		StringBuilder result = new StringBuilder();

		result.append(Integer.toString(instanceId[0] & 0xff));
		for (int i = 1; i < instanceId.length; i++) {
			result.append(".").append(Integer.toString(instanceId[i] & 0xff));
		}

		return result.toString();
	}

	public int getClassId() {
		return classId;
	}

	public String getInstanceId() {
		return instanceId;
	}

	public int getAttributeId() {
		return attributeId;
	}

	public GetParameter createGetParameter() {
		return new GetParameter(classId, instanceId, attributeId);
	}

	@Override
	public String toString() {
		return classId + "/" + printableInstanceId + "/" + attributeId;
	}
}
