/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openmuc.framework.driver.knx;

import java.io.IOException;
import java.net.InetAddress;
import java.util.LinkedList;
import java.util.List;

import org.openmuc.framework.config.DeviceScanInformation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import tuwien.auto.calimero.knxnetip.Discoverer;
import tuwien.auto.calimero.knxnetip.servicetype.SearchResponse;

/**
 * @author frobra
 * 
 */
public class KnxIpDiscover {

	private static Logger logger = LoggerFactory.getLogger(KnxIpDiscover.class);
	private static int DEFALUT_TIMEOUT = 5;

	private Discoverer discoverer;
	private SearchResponse[] searchResponses;

	public KnxIpDiscover(String interfaceAddress, boolean natAware, boolean mcastResponse) throws IOException {
		try {
			// System.setProperty("java.net.preferIPv4Stack", "true");
			InetAddress localHost = InetAddress.getByName(interfaceAddress);
			discoverer = new Discoverer(localHost, 0, natAware, mcastResponse);
		} catch (Exception e) {
			logger.warn("can not create discoverer: " + e.getMessage());
			throw new IOException(e);
		}
	}

	public boolean startSearch(int timeout) throws IOException {
		timeout = timeout / 1000;
		if (timeout < 1) {
			timeout = DEFALUT_TIMEOUT;
		}
		try {
			logger.debug("Starting search (timeout: " + timeout + "s)");
			discoverer.startSearch(timeout, true);
			searchResponses = discoverer.getSearchResponses();
		} catch (Exception e) {
			logger.warn("A network I/O error occurred");
			e.printStackTrace();
			throw new IOException(e);
		}
		return (searchResponses != null);
	}

	public List<DeviceScanInformation> getResponses() {
		List<DeviceScanInformation> informations = new LinkedList<DeviceScanInformation>();

		for (SearchResponse response : searchResponses) {
			StringBuilder deviceAddress = new StringBuilder();
			deviceAddress.append(KnxDriver.ADDRESS_SCHEME_KNXIP).append("://");
			String ipAddress = response.getControlEndpoint().getAddress().getHostAddress();
			if (ipAddress.contains(":")) { // if it is an ipv6 address
				deviceAddress.append("[").append(ipAddress).append("]");
			}
			else {
				deviceAddress.append(ipAddress);
			}
			deviceAddress.append(":").append(response.getControlEndpoint().getPort());

			String name = response.getDevice().getSerialNumberString();
			String description = response.getDevice().toString();

			logger.debug("Found " + deviceAddress + " - " + name + " - " + description);

			informations.add(new DeviceScanInformation(deviceAddress.toString(), name, description));
		}

		return informations;
	}
}
