/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openmuc.framework.server.restws.channel;

import java.io.IOException;
import java.util.List;

import org.openmuc.framework.data.Record;
import org.openmuc.framework.dataaccess.Channel;
import org.openmuc.framework.dataaccess.DataLoggerNotAvailableException;
import org.restlet.data.MediaType;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.Get;
import org.restlet.resource.ServerResource;

public final class ChannelHistoryResource extends ServerResource {

	/**
	 * Gets the history of records from a single channel
	 * 
	 * @return A StringRepresentation containing the JSON string
	 * @throws DataLoggerNotAvailableException
	 * @throws IOException
	 */
	@Get("json")
	public Representation getHistory() throws DataLoggerNotAvailableException, IOException {
		Channel channel = WebserviceProvider.getChannel(getRequest(), getResponse());
		if (channel == null) {
			return null;
		}

		String from = getQueryValue("from");
		String until = getQueryValue("until");

		if (from == null) {
			getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			return null;
		}

		long fromTimestamp = 0;
		try {
			fromTimestamp = Long.parseLong(from);
		} catch (NumberFormatException ex) {
			getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			return null;
		}

		Long untilTimestamp = null;
		List<Record> records = null;

		if (until == null) {
			records = channel.getLoggedValues(fromTimestamp);
		}
		else {
			try {
				untilTimestamp = Long.parseLong(until);
				records = channel.getLoggedValues(fromTimestamp, untilTimestamp);
			} catch (NumberFormatException ex) {
				getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
				return null;
			}
		}

		JsonWrapper wrapper = WebserviceProvider.getWrapper();
		return new StringRepresentation(wrapper.fromRecordList(records), MediaType.APPLICATION_JSON);
	}
}
