/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openmuc.framework.server.restws.channel;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.openmuc.framework.dataaccess.Channel;
import org.openmuc.framework.dataaccess.DataAccessService;
import org.restlet.data.MediaType;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.Get;
import org.restlet.resource.ServerResource;

import com.fasterxml.jackson.core.JsonProcessingException;

/**
 * Webservice for listing all available channels
 * 
 * @author Karsten Mueller-Bier
 */
public final class ChannelListResource extends ServerResource {

	/**
	 * Creates a JSON List of all channels
	 * 
	 * @return StringRepresentation containing the JSON string
	 * @throws JsonProcessingException
	 *             If there was a problem with creating the JSON string
	 */
	@Get("json")
	public Representation listJson() throws IOException {
		DataAccessService dataAccess = WebserviceProvider.getDataAccess();

		List<String> labels = dataAccess.getAllLabels();
		List<Channel> channels = new ArrayList<Channel>(labels.size());

		for (String label : labels) {
			channels.add(dataAccess.getChannel(label));
		}

		JsonWrapper wrapper = WebserviceProvider.getWrapper();
		return new StringRepresentation(wrapper.fromChannelList(channels), MediaType.APPLICATION_JSON);
	}
}
