/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.openmuc.framework.webui.channelconfigurator;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.openmuc.framework.config.ChannelConfig;
import org.openmuc.framework.config.DeviceConfig;
import org.openmuc.framework.config.DriverConfig;
import org.openmuc.framework.data.ValueType;
import org.openmuc.framework.webui.spi.ResourceLoader;
import org.osgi.service.component.ComponentContext;

public final class WebconfigViewFactory {

	private final ResourceLoader loader;

	public WebconfigViewFactory(ComponentContext context) {
		loader = new ResourceLoader(context.getBundleContext());
	}

	public WebconfigView createDriverView(Collection<DriverConfig> driverList) {
		WebconfigView view = new WebconfigView("Driver List", loader.getResourceAsString("drivers.html"));

		view.addToContext("driverConfigs", driverList);
		return view;
	}

	public WebconfigView createEditDriverView(DriverConfig driver, String title) {
		WebconfigView view = new WebconfigView("Edit Driver", loader.getResourceAsString("editdriver.html"));

		view.addToContext("title", title);
		view.addToContext("driver", driver);
		return view;
	}

	public WebconfigView createDeleteDriverView(DriverConfig driver) {
		WebconfigView view = new WebconfigView("Confirm delete driver", loader.getResourceAsString("deletedriver.html"));

		view.addToContext("driver", driver);
		return view;
	}

	public WebconfigView createDeviceView(Collection<DriverConfig> driverList) {
		return createDeviceView(driverList, "Devices");
	}

	public WebconfigView createDeviceView(Collection<DriverConfig> driverList, String title) {
		WebconfigView view = new WebconfigView("Device List", loader.getResourceAsString("devices.html"));

		if ("Devices".equals(title) == false) {
			view.addToContext("extraTab", true);
		}
		view.addToContext("title", title);
		view.addToContext("driverList", driverList);
		return view;
	}

	public WebconfigView createEditDeviceView(String title, Collection<DriverConfig> driverList, DeviceConfig device) {
		WebconfigView view = new WebconfigView("Edit Device", loader.getResourceAsString("editdevice.html"));

		view.addToContext("title", title);
		view.addToContext("driverList", driverList);
		view.addToContext("device", device);
		return view;
	}

	public WebconfigView createDeleteDeviceView(String driverId, DeviceConfig device) {
		WebconfigView view;
		view = new WebconfigView("Confirm Delete Device", loader.getResourceAsString("deletedevice.html"));

		view.addToContext("driverId", driverId);
		view.addToContext("device", device);
		return view;
	}

	public WebconfigView createChannelsView(String driverId, DeviceConfig device) {
		WebconfigView view = new WebconfigView("Show Channels", loader.getResourceAsString("channels.html"));

		view.addToContext("extraTab", true);
		view.addToContext("title", "Channels of device " + device.getName());
		view.addToContext("driverList", Arrays.asList(new DriverConfigMock(driverId, device)));
		return view;
	}

	public WebconfigView createChannelsView(Collection<DriverConfig> driverList) {
		WebconfigView view = new WebconfigView("Channel List", loader.getResourceAsString("channels.html"));

		view.addToContext("title", "Channels");
		view.addToContext("driverList", driverList);
		return view;
	}

	public WebconfigView createEditChannelView(String title, String driverId, DeviceConfig device, ChannelConfig channel) {
		WebconfigView view = new WebconfigView("Edit Channel", loader.getResourceAsString("editchannel.html"));

		view.addToContext("title", title);
		view.addToContext("driverId", driverId);
		view.addToContext("device", device);
		view.addToContext("values", ValueType.values());
		view.addToContext("channel", channel);
		return view;
	}

	public WebconfigView createMessage(String title, String text) {
		WebconfigView view = new WebconfigView("Error", loader.getResourceAsString("message.html"));

		view.addToContext("Title", title);
		view.addToContext("Message", text);
		return view;
	}

	public WebconfigView createFileOptions() {
		WebconfigView view = new WebconfigView("File Options", loader.getResourceAsString("fileoptions.html"));

		return view;
	}

	public WebconfigView createScanChannelsView(String title) {
		WebconfigView view = new WebconfigView("Scanned channels", loader.getResourceAsString("scanchannels.html"));

		view.addToContext("extraTab", true);
		view.addToContext("title", title);
		return view;
	}

	public static class DriverConfigMock {

		private final List<DeviceConfig> devices = new LinkedList<DeviceConfig>();
		private final String id;

		public DriverConfigMock(String id, DeviceConfig device) {
			this.id = id;
			devices.add(device);
		}

		public Collection<DeviceConfig> getDevices() {
			return devices;
		}

		public String getId() {
			return id;
		}
	}

}
